@@ -8,6 +8,7 @@ Response objects used throughout the project.
 ----
 """
 
+from typing import Generic, List
 from .utils import generate_token
 from .errors import (
     InvalidClientError,
@@ -16,28 +17,28 @@ from .errors import (
     UnsupportedResponseTypeError,
 )
 from .models import Client
-from .requests import Request
+from .requests import TRequest
 from .responses import (
     AuthorizationCodeResponse,
     IdTokenResponse,
     NoneResponse,
     TokenResponse,
 )
-from .storage import BaseStorage
+from .storage import TStorage
 from .types import CodeChallengeMethod
 
 
-class ResponseTypeBase:
+class ResponseTypeBase(Generic[TRequest, TStorage]):
     """Base response type that all other exceptions inherit from."""
 
-    def __init__(self, storage: BaseStorage):
+    def __init__(self, storage: TStorage):
         self.storage = storage
 
-    async def validate_request(self, request: Request) -> Client:
-        code_challenge_methods = list(CodeChallengeMethod)
+    async def validate_request(self, request: TRequest) -> Client:
+        code_challenge_methods: List[CodeChallengeMethod] = ["plain", "S256"]
 
         if not request.query.client_id:
-            raise InvalidRequestError(
+            raise InvalidRequestError[TRequest](
                 request=request, description="Missing client_id parameter."
             )
 
@@ -46,50 +47,50 @@ class ResponseTypeBase:
         )
 
         if not client:
-            raise InvalidRequestError(
+            raise InvalidRequestError[TRequest](
                 request=request, description="Invalid client_id parameter value."
             )
 
         if not request.query.redirect_uri:
-            raise InvalidRequestError(
+            raise InvalidRequestError[TRequest](
                 request=request, description="Mismatching redirect URI."
             )
 
         if not client.check_redirect_uri(request.query.redirect_uri):
-            raise InvalidRequestError(
+            raise InvalidRequestError[TRequest](
                 request=request, description="Invalid redirect URI."
             )
 
         if request.query.code_challenge_method:
             if request.query.code_challenge_method not in code_challenge_methods:
-                raise InvalidRequestError(
+                raise InvalidRequestError[TRequest](
                     request=request, description="Transform algorithm not supported."
                 )
 
             if not request.query.code_challenge:
-                raise InvalidRequestError(
+                raise InvalidRequestError[TRequest](
                     request=request, description="Code challenge required."
                 )
 
         if not client.check_response_type(request.query.response_type):
-            raise UnsupportedResponseTypeError(request=request)
+            raise UnsupportedResponseTypeError[TRequest](request=request)
 
         if not client.check_scope(request.query.scope):
-            raise InvalidScopeError(request=request)
+            raise InvalidScopeError[TRequest](request=request)
 
         if not request.user:
-            raise InvalidClientError(
+            raise InvalidClientError[TRequest](
                 request=request, description="User is not authorized"
             )
 
         return client
 
 
-class ResponseTypeToken(ResponseTypeBase):
+class ResponseTypeToken(ResponseTypeBase[TRequest, TStorage]):
     """Response type that contains a token."""
 
     async def create_authorization_response(
-        self, request: Request, client: Client
+        self, request: TRequest, client: Client
     ) -> TokenResponse:
         token = await self.storage.create_token(
             request,
@@ -108,11 +109,11 @@ class ResponseTypeToken(ResponseTypeBase):
         )
 
 
-class ResponseTypeAuthorizationCode(ResponseTypeBase):
+class ResponseTypeAuthorizationCode(ResponseTypeBase[TRequest, TStorage]):
     """Response type that contains an authorization code."""
 
     async def create_authorization_response(
-        self, request: Request, client: Client
+        self, request: TRequest, client: Client
     ) -> AuthorizationCodeResponse:
         authorization_code = await self.storage.create_authorization_code(
             request,
@@ -130,20 +131,20 @@ class ResponseTypeAuthorizationCode(ResponseTypeBase):
         )
 
 
-class ResponseTypeIdToken(ResponseTypeBase):
-    async def validate_request(self, request: Request) -> Client:
+class ResponseTypeIdToken(ResponseTypeBase[TRequest, TStorage]):
+    async def validate_request(self, request: TRequest) -> Client:
         client = await super().validate_request(request)
 
         # nonce is required for id_token
         if not request.query.nonce:
-            raise InvalidRequestError(
+            raise InvalidRequestError[TRequest](
                 request=request,
                 description="Nonce required for response_type id_token.",
             )
         return client
 
     async def create_authorization_response(
-        self, request: Request, client: Client
+        self, request: TRequest, client: Client
     ) -> IdTokenResponse:
         id_token = await self.storage.get_id_token(
             request,
@@ -157,8 +158,8 @@ class ResponseTypeIdToken(ResponseTypeBase):
         return IdTokenResponse(id_token=id_token)
 
 
-class ResponseTypeNone(ResponseTypeBase):
+class ResponseTypeNone(ResponseTypeBase[TRequest, TStorage]):
     async def create_authorization_response(
-        self, request: Request, client: Client
+        self, request: TRequest, client: Client
     ) -> NoneResponse:
         return NoneResponse()
